@@ -151,20 +151,6 @@ class Api(object):
         except requests.RequestException:
             return False
 
-    def save_pip(self, out_dir):
-        """Saves the current working set of pip packages to requirements.txt"""
-        try:
-            import pkg_resources
-
-            installed_packages = [d for d in iter(pkg_resources.working_set)]
-            installed_packages_list = sorted(
-                ["%s==%s" % (i.key, i.version) for i in installed_packages]
-            )
-            with open(os.path.join(out_dir, 'requirements.txt'), 'w') as f:
-                f.write("\n".join(installed_packages_list))
-        except Exception as e:
-            logger.error("Error saving pip packages")
-
     def save_patches(self, out_dir):
         """Save the current state of this repository to one or more patches.
 
