@@ -254,23 +254,15 @@ class Api(object):
         self._runs = {}
 
     def _parse_project_path(self, path):
-        """Parses paths in the following formats:
-
-        entity/project
-        project
-
-        entity is optional and will fall back to the current logged in user
-        """
-        parts = path.split("/")
-        entity = self.settings['entity'] or self.default_entity
+        """Returns project and entity for project specified by path"""
         project = self.settings['project']
+        entity = self.settings['entity'] or self.default_entity
+        if path is None:
+            return entity, project
+        parts = path.split('/', 1)
         if len(parts) == 1:
-            project = parts[1]
-        elif len(parts) == 2:
-            entity, project = parts[0], parts[1]
-        else:
-            raise ValueError('Invalid project path: %s' % path)
-        return entity, project
+            return entity, path
+        return parts
 
     def _parse_path(self, path):
         """Parses paths in the following formats:
@@ -317,17 +309,6 @@ class Api(object):
             return entity, parts[0], parts[1]
         return parts
 
-    def _parse_project_path(self, path):
-        """Returns project and entity for project specified by path"""
-        project = self.settings['project']
-        entity = self.settings['entity'] or self.default_entity
-        if path is None:
-            return entity, project
-        parts = path.split('/', 1)
-        if len(parts) == 1:
-            return entity, path
-        return parts
-
     def projects(self, entity=None, per_page=200):
         """Get projects for a given entity.
         Args:
@@ -2339,6 +2320,8 @@ class Artifact(object):
             raise ValueError('Project %s/%s does not contain artifact: "%s"' % (
                 self.entity, self.project, self.artifact_name))
         self._attrs = response['project']['artifact']
+        if 'metadata' in response['project']['artifact'] and response['project']['artifact']['metadata']:
+            self._metadata = json.loads(response['project']['artifact']['metadata'])
         return self._attrs
 
     # The only file should be wandb_manifest.json
