@@ -81,8 +81,11 @@ class BackendSender(object):
             print("unknown type")
         o = wandb_internal_pb2.OutputData(output_type=otype, line=data)
         o.timestamp.GetCurrentTime()
+        self._send_output(o)
+
+    def _send_output(self, outdata):
         rec = wandb_internal_pb2.Record()
-        rec.output.CopyFrom(o)
+        rec.output.CopyFrom(outdata)
         self._queue_process(rec)
 
     def _send_history(self, history):
@@ -288,11 +291,17 @@ class BackendSender(object):
 
     def send_config(self, config_dict):
         cfg = self._make_config(config_dict)
+        self._send_config(cfg)
+
+    def _send_config(self, cfg):
         rec = self._make_record(config=cfg)
         self._queue_process(rec)
 
     def send_summary(self, summary_dict):
         summary = self._make_summary(summary_dict)
+        self._send_summary(summary)
+
+    def _send_summary(self, summary):
         rec = self._make_record(summary=summary)
         self._queue_process(rec)
 
@@ -309,7 +318,8 @@ class BackendSender(object):
 
         req = self._make_record(run=run)
         resp = self._request_response(req, timeout=timeout)
-        return resp
+        assert resp.run_result
+        return resp.run_result
 
     def send_run_sync(self, run_obj, timeout=None):
         run = self._make_run(run_obj)
@@ -347,7 +357,8 @@ class BackendSender(object):
         req = self._make_record(exit=exit_data)
 
         resp = self._request_response(req, timeout=timeout)
-        return resp
+        assert resp.exit_result
+        return resp.exit_result
 
     def send_exit_sync(self, exit_code, timeout=None):
         exit_data = self._make_exit(exit_code)
