@@ -153,20 +153,6 @@ class Api(object):
         except requests.RequestException:
             return False
 
-    def save_pip(self, out_dir):
-        """Saves the current working set of pip packages to requirements.txt"""
-        try:
-            import pkg_resources
-
-            installed_packages = [d for d in iter(pkg_resources.working_set)]
-            installed_packages_list = sorted(
-                ["%s==%s" % (i.key, i.version) for i in installed_packages]
-            )
-            with open(os.path.join(out_dir, 'requirements.txt'), 'w') as f:
-                f.write("\n".join(installed_packages_list))
-        except Exception as e:
-            logger.error("Error saving pip packages")
-
     def save_patches(self, out_dir):
         """Save the current state of this repository to one or more patches.
 
@@ -236,8 +222,9 @@ class Api(object):
         if auth:
             key = auth[-1]
         # Environment should take precedence
-        if self._environ.get(env.API_KEY):
-            key = self._environ.get(env.API_KEY)
+        env_key = self._environ.get(env.API_KEY)
+        if env_key is not None:
+            key = env_key
         return key
 
     @property
@@ -407,8 +394,8 @@ class Api(object):
                 [{"id","name","repo","dockerImage","description"}]
         """
         query = gql('''
-        query Models($entity: String, $project: String!, $sweep: String!, $specs: [JSONString!]!) {
-            model(name: $project, entityName: $entity) {
+        query Sweep($entity: String, $project: String!, $sweep: String!, $specs: [JSONString!]!) {
+            project(name: $project, entityName: $entity) {
                 sweep(sweepName: $sweep) {
                     id
                     name
@@ -448,9 +435,9 @@ class Api(object):
         project = project or self.settings('project')
         response = self.gql(query, variable_values={'entity': entity,
                                                     'project': project, 'sweep': sweep, 'specs': specs})
-        if response['model'] is None or response['model']['sweep'] is None:
+        if response['project'] is None or response['project']['sweep'] is None:
             raise ValueError("Sweep {}/{}/{} not found".format(entity, project, sweep) )
-        data = response['model']['sweep']
+        data = response['project']['sweep']
         if data:
             data['runs'] = self._flatten_edges(data['runs'])
         return data
@@ -679,7 +666,9 @@ class Api(object):
         ''')
         response = self.gql(mutation, variable_values={
             'name': self.format_project(project), 'entity': entity or self.settings('entity'),
-            'description': description, 'repo': self.git.remote_url, 'id': id})
+            'description': description, 'id': id})
+        # TODO(jhr): Commenting out 'repo' field for cling, add back
+        #   'description': description, 'repo': self.git.remote_url, 'id': id})
         return response['upsertModel']['model']
 
     @normalize_exceptions
@@ -790,7 +779,8 @@ class Api(object):
             kwargs['num_retries'] = num_retries
 
         variable_values = {
-            'id': id, 'entity': entity or self.settings('entity'), 'name': name, 'project': project,
+            'id': id, 'entity': entity or self.settings('entity'), 'name': name,
+            'project': project or util.auto_project_name(program_path),
             'groupName': group, 'tags': tags,
             'description': description, 'config': config, 'commit': commit,
             'displayName': display_name, 'notes': notes,
@@ -1058,7 +1048,7 @@ class Api(object):
         def no_retry_4xx(e):
             if not isinstance(e, requests.HTTPError):
                 return True
-            if not(e.response.status_code >= 400 and e.response.status_code < 500):
+            if not(e.response.status_code >= 400 and e.response.status_code < 500) or e.response.status_code == 429:
                 return True
             body = json.loads(e.response.content)
             raise UsageError(body['errors'][0]['message'])
@@ -1164,7 +1154,7 @@ class Api(object):
         def no_retry_4xx(e):
             if not isinstance(e, requests.HTTPError):
                 return True
-            if not(e.response.status_code >= 400 and e.response.status_code < 500):
+            if not(e.response.status_code >= 400 and e.response.status_code < 500) or e.response.status_code == 429:
                 return True
             body = json.loads(e.response.content)
             raise UsageError(body['errors'][0]['message'])
