@@ -96,7 +96,6 @@ class _WandbInit(object):
             "force",
             "tensorboard",
             "sync_tensorboard",
-            "monitor_gym",
         )
         for key in unsupported:
             val = kwargs.pop(key, None)
@@ -105,6 +104,10 @@ class _WandbInit(object):
                     "currently unsupported wandb.init() arg: %s", key
                 )
 
+        monitor_gym = kwargs.pop("monitor_gym", None)
+        if monitor_gym and len(wandb.patched["gym"]) == 0:
+            wandb.gym.monitor()
+
         # prevent setting project, entity if in sweep
         # TODO(jhr): these should be locked elements in the future or at least
         #            moved to apply_init()
@@ -331,7 +334,7 @@ class _WandbInit(object):
             log_artifact=run.log_artifact,
         )
         self._reporter.set_context(run=run)
-        run.on_start()
+        run._on_start()
 
         return run
 
