@@ -19,6 +19,7 @@ import traceback
 
 import click
 from six import string_types
+from six.moves.urllib.parse import quote as url_quote
 import wandb
 from wandb.apis import internal, public
 from wandb.data_types import _datatypes_set_callback
@@ -119,6 +120,7 @@ class RunManaged(Run):
         self._save_stderr = None
         self._stdout_slave_fd = None
         self._stderr_slave_fd = None
+        self._exit_code = None
 
         # Pull info from settings
         self._init_from_settings(settings)
@@ -199,6 +201,24 @@ class RunManaged(Run):
             return None
         return self._run_obj.display_name
 
+    @name.setter
+    def name(self, name):
+        self._name = name
+        if self._backend:
+            self._backend.interface.send_run(self)
+
+    @property
+    def notes(self):
+        if not self._run_obj:
+            return None
+        return self._run_obj.notes
+
+    @notes.setter
+    def notes(self, notes):
+        self._notes = notes
+        if self._backend:
+            self._backend.interface.send_run(self)
+
     @property
     def id(self):
         return self._run_id
@@ -426,6 +446,9 @@ class RunManaged(Run):
         if not isinstance(data, collections.Mapping):
             raise ValueError("wandb.log must be passed a dictionary")
 
+        if any(not isinstance(key, string_types) for key in data.keys()):
+            raise ValueError("Key values passed to `wandb.log` must be strings.")
+
         if step is not None:
             if self.history._step > step:
                 wandb.termwarn(
@@ -585,14 +608,16 @@ class RunManaged(Run):
         s = self._settings
         r = self._run_obj
         app_url = s.base_url.replace("//api.", "//app.")
-        url = "{}/{}/{}".format(app_url, r.entity, r.project)
+        url = "{}/{}/{}".format(app_url, url_quote(r.entity), url_quote(r.project))
         return url
 
     def _get_run_url(self):
         s = self._settings
         r = self._run_obj
         app_url = s.base_url.replace("//api.", "//app.")
-        url = "{}/{}/{}/runs/{}".format(app_url, r.entity, r.project, r.run_id)
+        url = "{}/{}/{}/runs/{}".format(
+            app_url, url_quote(r.entity), url_quote(r.project), url_quote(r.run_id)
+        )
         return url
 
     def _get_run_name(self):
@@ -695,17 +720,10 @@ class RunManaged(Run):
 
         exit_code = exit_code or self._hooks.exit_code if self._hooks else 0
         logger.info("got exitcode: %d", exit_code)
-        ret = self._backend.interface.send_exit_sync(exit_code, timeout=60)
-        logger.info("got exit ret: %s", ret)
-        if ret is None:
-            print("Problem syncing data")
-            os._exit(1)
 
-        #  TODO: close the logging file handler
-
-        self.on_finish()
-
-        self.on_final()
+        self._exit_code = exit_code
+        self._on_finish()
+        self._on_final()
 
     def _console_start(self):
         logger.info("atexit reg")
@@ -723,7 +741,7 @@ class RunManaged(Run):
     def _console_stop(self):
         self._restore()
 
-    def on_start(self):
+    def _on_start(self):
         wandb.termlog("Tracking run with wandb version {}".format(wandb.__version__))
         if self._run_obj:
             run_state_str = "Syncing run"
@@ -735,12 +753,21 @@ class RunManaged(Run):
         print("")
         self._console_start()
 
-    def on_finish(self):
+    def _on_finish(self):
+        # make sure all uncommitted history is flushed
+        self.history._flush()
+
+        ret = self._backend.interface.send_exit_sync(self._exit_code, timeout=60)
+        logger.info("got exit ret: %s", ret)
+        if ret is None:
+            print("Problem syncing data")
+            os._exit(1)
+
+        #  TODO: close the logging file handler
         self._console_stop()
         self._backend.cleanup()
-        pass
 
-    def on_final(self):
+    def _on_final(self):
         # check for warnings and errors, show log file locations
         # if self._run_obj:
         #    self._display_run()
@@ -840,7 +867,9 @@ class RunManaged(Run):
             name = artifact_or_name
             if type is None:
                 raise ValueError("type required")
-            public_api = public.Api()
+            public_api = public.Api(
+                {"entity": r.entity, "project": r.project, "run": self.id}
+            )
             artifact = public_api.artifact(type=type, name=name)
             if type is not None and type != artifact.type:
                 raise ValueError(
@@ -927,3 +956,11 @@ class RunManaged(Run):
         self._use_redirect = use_redirect
         self._stdout_slave_fd = stdout_slave_fd
         self._stderr_slave_fd = stderr_slave_fd
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        exit_code = 0 if exc_type is None else 1
+        self.join(exit_code)
+        return exc_type is None
